
import entities.Inventory;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;
import javax.persistence.TypedQuery;

public class Main {

    private static EntityManager em;

    public static void main(String[] args) {
        withDbTransaction(() -> {
            deleteRecords();

            insertRecords();
            selectRecords();

            updateRecords();
            selectRecords();
        });
    }

    private static void withDbTransaction(Runnable transaction) {
        // Get an entity manager factory by persistence unit name
        // The database table will be automatically created by eclipselink
        EntityManagerFactory emf = Persistence.createEntityManagerFactory("testpu");

        // Create an entity manager
        em = emf.createEntityManager();

        // Begin transaction
        em.getTransaction().begin();

        // The caller has specified what to do, run those steps now...
        transaction.run();

        // Commit the transaction and close the entity manager
        em.getTransaction().commit();
        em.close();
    }

    private static void deleteRecords() {
        // Bulk delete with criteria API
        Query query = em.createQuery("DELETE FROM Inventory");
        int removedItems = query.executeUpdate();

        if (removedItems > 0) {
            System.out.println(removedItems + " items removed.");
        }
    }

    private static void insertRecords() {
        for(int i = 0; i < 5; ++i) {
            insertRecord(i);
        }
    }

	private static void insertRecord(int i) {
		Inventory inv = makeInventory(i);
		em.persist(inv);
	}

	private static Inventory makeInventory(int i) {
		Inventory inv = new Inventory();
		inv.setName("Item #" + i);
		inv.setQuantity(i + 2);
		inv.setPrice(i * 1.41f);
		return inv;
	}

    private static void selectRecords() {
        // Select with JPQL and print out entities
        TypedQuery<Inventory> q = em.createQuery("SELECT i FROM Inventory AS i ORDER BY i.name", Inventory.class);
        List<Inventory> items = q.getResultList();

        System.out.println();

        for(Inventory i : items) {
            System.out.println(i);
        }

        System.out.println();

        // Clear the entity cache to force re-select next time
        em.clear();
    }

    private static void updateRecords() {
        System.out.println("Updating items with prirce 0 to 100...");

        // Update with criteria API
        Query query = em.createQuery("UPDATE Inventory AS i SET i.price = 100.0 WHERE i.price = 0.0");
        int updatedItems = query.executeUpdate();

        if (updatedItems > 0) {
            System.out.println(updatedItems + " items updated.");
        }
    }
}
