
package entities;

import java.io.Serializable;
import java.util.LinkedList;
import java.util.List;
import javax.persistence.*;

@Entity
public class Employee implements Serializable {
    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    private int id;

    @Column(length = 100)
    private String name;

    // Inverse side of a one-to-one relationship
    @OneToOne(mappedBy = "employee", cascade = CascadeType.ALL)
    private ParkingLot parkingLot;

    // Owning side of a many-to-many relationship
    @ManyToOne(cascade = CascadeType.PERSIST)
    // Rename the joining column added to this table
    @JoinColumn(name = "dept_id", nullable = false)
    private Department department;

    // "Owning side" of a many-to-many relaionship
    @ManyToMany(cascade = CascadeType.PERSIST)
    // Rename the joining table and its columns created for the relation
    @JoinTable(name="scheduling",
        joinColumns = @JoinColumn(name = "emp_id"),
        inverseJoinColumns = @JoinColumn(name = "met_id"))
    private List<Meeting> meetings;

    public Employee() {
        meetings = new LinkedList<>();
    }

    public int getId() {
        return id;
    }

    public void setId(int id) {
        this.id = id;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public ParkingLot getParkingLot() {
        return parkingLot;
    }

    public void setParkingLot(ParkingLot parkingLot) {
        this.parkingLot = parkingLot;
    }

    public Department getDepartment() {
        return department;
    }

    public void setDepartment(Department department) {
        this.department = department;
    }

    public List<Meeting> getMeetings() {
        return meetings;
    }

    public void setMeetings(List<Meeting> meetings) {
        this.meetings = meetings;
    }

    public void addMeeting(Meeting m) {
        this.meetings.add(m);
    }

    @Override
    public int hashCode() {
        int hash = 0;
        hash += (int) id;
        return hash;
    }

    @Override
    public boolean equals(Object object) {
        // TODO: Warning - this method won't work in the case the id fields are not set
        if (!(object instanceof Employee)) {
            return false;
        }
        Employee other = (Employee) object;
        if (this.id != other.id) {
            return false;
        }
        return true;
    }

    @Override
    public String toString() {
        return "entities.Employee[ id=" + id + ", name=" + name + " ]";
    }
}
