

import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.ExportException;
import java.rmi.server.UnicastRemoteObject;
import java.util.logging.Level;
import java.util.logging.Logger;

public class ServerMain {

    public static void main(String[] args) {
        try {
            // Instantiate the service implementation
            CalculatorServiceImpl service = new CalculatorServiceImpl();

            // Export it on a random port (start to listen to clients)
            // 0 means that the system will decide the object's port
            Remote stub = UnicastRemoteObject.exportObject(service, 0);

            Registry registry = getRegistry();

            // Make the service available through a fixed name
            registry.rebind(CalculatorService.NAME, stub);

            // Indicate that the server started successfully
            System.err.println("Server started ...");
        } catch (RemoteException ex) {
            Logger.getLogger(ServerMain.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

	// If an RMI registry is available, use it.
    // Otherwise, create one ourselves on the default port.
	private static Registry getRegistry() throws RemoteException {
		boolean isCreated = false;
		try {
			isCreated = true;
			return LocateRegistry.createRegistry(1099);
		} catch (ExportException e) {
			return LocateRegistry.getRegistry();
		} finally {
			System.out.println("Registry is " + (isCreated ? "created." : "found."));
		}
	}
}
