
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.logging.Level;
import java.util.logging.Logger;

// Note that the service caller code is almost duplicated in this version.
public class ClientMain {

    private static final String HOST = "localhost";
    private static final int PORT = 1234;

    public static void main(String[] args) {
        System.out.println("Addition result: " + callAdd(10, 20));
        System.out.println("Square root result: " + callSqrt(2));
    }

    private static int callAdd(int a, int b) {
        Socket s = null;

        try {
            // Open connection to the server
            s = new Socket(HOST, PORT);

            // Send request: write in the method name and the operands
            OutputStream os = s.getOutputStream();
            ObjectOutputStream oos = new ObjectOutputStream(os);
            oos.writeUTF("add");
            oos.writeInt(a);
            oos.writeInt(b);
            oos.flush(); // important: immediately write contents to the socket

            // Receive response: read out the addition result
            InputStream is = s.getInputStream();
            ObjectInputStream ois = new ObjectInputStream(is);
            int response = ois.readInt();

            // Return response
            return response;
        } catch (UnknownHostException ex) {
            Logger.getLogger(ClientMain.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(ClientMain.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            // Close connection to the server
            if(s != null) {
                try {
                    s.close();
                } catch (IOException ex) {
                    Logger.getLogger(ClientMain.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        }

        // Fall-back value in the case of an exception
        return 0;
    }

    private static double callSqrt(double a) {
        Socket s = null;

        try {
            // Open connection to the server
            s = new Socket(HOST, PORT);

            // Send request: write in the method name and the operands
            OutputStream os = s.getOutputStream();
            ObjectOutputStream oos = new ObjectOutputStream(os);
            oos.writeUTF("sqrt");
            oos.writeDouble(a);
            oos.flush(); // important: immediately write contents to the socket

            // Receive response: read out the addition result
            InputStream is = s.getInputStream();
            ObjectInputStream ois = new ObjectInputStream(is);
            double response = ois.readDouble();

            // Return response
            return response;
        } catch (UnknownHostException ex) {
            Logger.getLogger(ClientMain.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(ClientMain.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            // Close connection to the server
            if(s != null) {
                try {
                    s.close();
                } catch (IOException ex) {
                    Logger.getLogger(ClientMain.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        }

        // Fall-back value in the case of an exception
        return 0;
    }
}
