
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.logging.Level;
import java.util.logging.Logger;

public class ServerMain {

    private static final int PORT = 1234;

    public static void main(String[] args) {
        try (
            // Open listener socket
            ServerSocket server = new ServerSocket(PORT);
        ) {
            // Indicate startup
            System.out.println("Server started to listen on port " + PORT);

            // Run the service continuously
            while(true) {
                // Look for a client connection, accept when found one
                Socket client = server.accept();

                // Process it on a separate thread
                ServerThread st = new ServerThread(client);
                st.start();
            }
        } catch (IOException ex) {
            Logger.getLogger(ServerMain.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    private static class ServerThread extends Thread {

        private Socket client;

        public ServerThread(Socket client) {
            this.client = client;
        }

        @Override
        public void run() {
            try {
                // Receive request: read out the method name
                InputStream is = client.getInputStream();
                ObjectInputStream ois = new ObjectInputStream(is);
                String methodName = ois.readUTF();

                // Create a service implementation instance
                CalculatorServiceImpl impl = new CalculatorServiceImpl();

                // Create the output stream
                OutputStream os = client.getOutputStream();
                ObjectOutputStream oos = new ObjectOutputStream(os);

                // Choose service operation
                switch (methodName) {
                    case "add":
                        {
                            // Read out the operands and invoke the service
                            int a = ois.readInt();
                            int b = ois.readInt();
                            int result = impl.add(a, b);
                            // Send response: write in the result
                            oos.writeInt(result);
                            break;
                        }
                    case "sqrt":
                        {
                            // Read out the operands and invoke the service
                            double a = ois.readDouble();
                            double result = impl.sqrt(a);
                            // Send response: write in the result
                            oos.writeDouble(result);
                            break;
                        }
                    default:
                        // In this case, the client wanted to use an operation
                        // which is not defined by the service interface.
                        break;
                }

                oos.flush(); // important: immediately write contents to the socket
            } catch (IOException ex) {
                Logger.getLogger(ServerMain.class.getName()).log(Level.SEVERE, null, ex);
            } finally {
                // Close connection to the client
                try {
                    client.close();
                } catch (IOException ex) {
                    Logger.getLogger(ServerMain.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        }
    }
}
