
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.logging.Level;
import java.util.logging.Logger;

public class ServerMain {

    private static final int PORT = 1234;

    public static void main(String[] args) {
        try (
            // Open listener socket
            ServerSocket server = new ServerSocket(PORT);
		) {
            // Indicate startup
            System.out.println("(server) Server started to listen on port " + PORT);

            // Run the service continuously
            while(true) {
                // Wait for a client connection (blocking operation)
                Socket client = server.accept();

                // Respond to the client
                respondTo(client);
            }
        } catch (IOException ex) {
            Logger.getLogger(ServerMain.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
   
    private static void respondTo(Socket client) {
        try {
            // Receive request
            InputStream is = client.getInputStream();
            InputStreamReader isr = new InputStreamReader(is);
            BufferedReader br = new BufferedReader(isr);

            String request = br.readLine();

            // Process request
            System.out.println("(server) Request: " + request);

            // Create response
            // Note that the response is fixed now,
            // but usually we use the request to construct it
            String response = "Another test message, from the server.";

            // Send response
            OutputStream os = client.getOutputStream();
            PrintWriter pw = new PrintWriter(os);
            pw.println(response);
            // important step: immediatelly write contents to the socket!
            // otherwise, pw can choose to keep the message in its cache
            pw.flush();
        } catch (IOException ex) {
            Logger.getLogger(ServerMain.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            // Close connection to the client
            try {
                client.close();
            } catch (IOException ex) {
                Logger.getLogger(ServerMain.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }
}
