package example;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.MessageConsumer;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

// Demonstrates synchronous message receiving through a JMS queue.
// It uses the common JMS API, which is available from version 1.1.
public class Main {

	public static void main(String[] args) throws NamingException, JMSException {
		// Lookup resources from the application server.
		Context ctx = new InitialContext();
		// Use the default JMS connection factory of GlassFish 
		ConnectionFactory connectionFactory = (ConnectionFactory)ctx.lookup(
			"java:comp/DefaultJMSConnectionFactory"
		);
		// The message destination is a custom message queue. This needs to be
		// set up in the administration console before running this application.
		Destination destination = (Destination)ctx.lookup("jms/ExampleQueue");
		ctx.close();
		
		// Create a connection
		try (Connection connection = connectionFactory.createConnection()) {
			// And an associated session
			Session session = connection.createSession();
			// With a message consumer
			MessageConsumer consumer = session.createConsumer(destination);
			// Start delivery of incoming messages
			connection.start();
			
			System.out.println("Receiving messages synchronously ...");
			
			// Receive and process messages continuously 
			while(true) {
				// The `receive` method synchronously receives a single message.
				// It will block the execution of the caller thread until a
				// message arrives. The web application sends only text messages
				// to the queue, and all of them have a `SenderUserAgent`
				// property.
				TextMessage message = (TextMessage)consumer.receive();
				System.out.println("Message received: `"
						+ message.getText() + "` (user agent: "
						+ message.getStringProperty("SenderUserAgent") + ")");
			}
		}
	}

}
