package example;

import javax.ejb.ActivationConfigProperty;
import javax.ejb.EJBException;
import javax.ejb.MessageDriven;
import javax.jms.JMSConnectionFactoryDefinition;
import javax.jms.JMSDestinationDefinition;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.TextMessage;

// Define a message driven session bean, which will be invoked automatically
// every time a message arrives to the given topic.

// To see all standard property names see:
// http://docs.oracle.com/javaee/7/api/javax/ejb/ActivationConfigProperty.html
@MessageDriven(
	activationConfig = {
		@ActivationConfigProperty(
			propertyName="destinationType",
			propertyValue="javax.jms.Topic"),
		// IMPORTANT NOTE: this is not the same as the `destination` property,
		// which defines the vendor-specific physical destination name!
		@ActivationConfigProperty(
			propertyName="destinationLookup",
			propertyValue="java:global/jms/ExampleTopic"),
		@ActivationConfigProperty(
			propertyName="connectionFactoryLookup",
			propertyValue="java:global/jms/AsyncEJBConnectionFactory")
})
// We define a new connection factory to provide a unique client id.
@JMSConnectionFactoryDefinition(
	name="java:global/jms/AsyncEJBConnectionFactory",
	interfaceName="javax.jms.ConnectionFactory",
	clientId="jmsejb-async"
)
//Define a JMS destination for this application.
@JMSDestinationDefinition(
	name="java:global/jms/ExampleTopic",
	interfaceName="javax.jms.Topic",
	destinationName="ExampleTopic"
)
public class AsyncMessageConsumerBean implements MessageListener {

    public void onMessage(Message m) {
    	try {
	    	if (m.getJMSRedelivered()) {
	    		// Do not process redelivered messages.
	    		return;
	    	}
    	
	    	// The web application sends only text messages to the
	    	// topic, and all of them have a `SenderUserAgent` property.
	    	TextMessage message = (TextMessage)m;
		
			System.out.printf("Message received: `%s` (user agent: %s)",
					message.getText(),
					message.getStringProperty("SenderUserAgent"));
		} catch (JMSException e) {
			e.printStackTrace();
			// Indicate the error to the container. The transaction will be
			// rolled back, so the message will be redelivered.
			throw new EJBException(e);
		}
    }

}
