package example;

import java.util.ArrayList;
import java.util.List;

import javax.annotation.Resource;
import javax.ejb.LocalBean;
import javax.ejb.Stateless;
import javax.inject.Inject;
import javax.jms.JMSConnectionFactory;
import javax.jms.JMSConnectionFactoryDefinition;
import javax.jms.JMSConsumer;
import javax.jms.JMSContext;
import javax.jms.JMSDestinationDefinition;
import javax.jms.Topic;

//Define a JMS connection factory for this application.
@JMSConnectionFactoryDefinition(
		name="java:global/jms/SyncEJBConnectionFactory",
		interfaceName="javax.jms.ConnectionFactory",
		clientId="jmsejb-sync"
	)
//Define a JMS destination for this application.
@JMSDestinationDefinition(
	name="java:global/jms/ExampleTopic",
	interfaceName="javax.jms.Topic",
	destinationName="ExampleTopic"
)
@Stateless
@LocalBean
public class SyncMessageConsumerBean {

	// Use the connection factory defined above.
	@Inject
	@JMSConnectionFactory(value="java:global/jms/SyncEJBConnectionFactory")
	private JMSContext context;
	
	// The message destination is a custom message topic. 
	@Resource(lookup="java:global/jms/ExampleTopic")
	private Topic destination;

	public List<String> getNewMessages() {
		List<String> result = new ArrayList<>();
		String value;
		
		// Create a subscription to the topic. It needs to be durable, because
		// the connection may not exist when the messages are sent to the topic.
		// It needs to be shared, because this way multiple bean instances can
		// use the same subscription. This means if a bean instance receives a
		// message, then the other instances will not receive that same message.
		
		// We also set a message selector expression which filters the messages,
		// so here we only receive the ones sent with a special value of the
		// SenderUserAgent property.
		JMSConsumer consumer = context.createSharedDurableConsumer(
			destination,
			"sync-consumer",
			"SenderUserAgent LIKE '%Firefox%'"
		);
		
		// Receive all filtered messages from the topic without blocking.
		// If the method returns null, there are no remaining messages in the
		// topic matching the message selector filter.
		// It also stops receiving after 10 messages.
		while (result.size() < 10
			   && (value = consumer.receiveBodyNoWait(String.class)) != null) {
			result.add(value);
		}
		
		return result;
	}
	
}
