package example;

import javax.annotation.Resource;
import javax.ejb.LocalBean;
import javax.ejb.Stateless;
import javax.inject.Inject;
import javax.jms.Destination;
import javax.jms.JMSConnectionFactory;
import javax.jms.JMSConnectionFactoryDefinition;
import javax.jms.JMSContext;
import javax.jms.JMSDestinationDefinition;

// A simple stateless EJB which can insert messages into a JMS topic.
// It uses the simplified JMS API, which is available from version 2.0.

// Because we use a JMS context to send the messages, which is not usable from
// a multi-threaded environment like a servlet's `doPost` method, we use this
// bean to create multiple contexts implicitly while the container creates more
// and more instances from it to serve its clients.

//Define a JMS connection factory for this application.
@JMSConnectionFactoryDefinition(
	name="java:global/jms/WebConnectionFactory",
	interfaceName="javax.jms.ConnectionFactory",
	clientId="jmsweb"
)
//Define a JMS destination for this application.
@JMSDestinationDefinition(
	name="java:global/jms/ExampleTopic",
	interfaceName="javax.jms.Topic",
	destinationName="ExampleTopic"
)
@Stateless
@LocalBean
public class MessageGeneratorBean {
	
	// Use the connection factory defined above.
	@Inject
	@JMSConnectionFactory(value="java:global/jms/WebConnectionFactory")
	private JMSContext context;
	
	// The message destination is a custom message topic. 
	@Resource(lookup="java:global/jms/ExampleTopic")
	private Destination destination;
	
	public void addMessage(String value, String senderUserAgent) {
		// Do not add empty messages to the queue
		if (value != null && value.trim().length() > 0) {
			// Create, assemble and send a new message in one step
			context.createProducer()
			       .setProperty("SenderUserAgent", senderUserAgent)
			       .send(destination, value);
		}
	}

}
