package example.stateless;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import javax.ejb.EJB;
import javax.ejb.Remote;
import javax.ejb.Stateless;

import example.stateless.Calculator;
import example.stateless.ValueHolder;

// A stateless session bean which implements the Calculator interface.
@Stateless(name="calcBean", mappedName="calc")
//The @Remote annotation specifies the methods of bean can be called remotely.
@Remote(Calculator.class)
public class CalculatorBean implements Calculator {

	// Injects a configured resource from the deployment descriptor.
	@Resource
	// Set a default value for the case when no value provided at deployment.
	private double sqrtErrorRate = 2;
	
	// Inject another stateless bean. We can use its class name because this
	// bean exposes a no-interface view. The container can inject it by its type.
	@EJB
	private SquareRootCalculatorBean squareRootHelper;
	
	// An interceptor method which is called after the dependency injection is done.
	@PostConstruct
	private void injectionDone() {
		// ...
	}
	
	@Override
	public ValueHolder add(ValueHolder a, ValueHolder b) {
		ValueHolder result = new ValueHolder(a.value + b.value);
		
		// Changing values of parameters will not have any effect in the client,
		// because they are passed by-copy through serialization.
		a.value = b.value = 0;
		
		return result;
	}

	@Override
	public ValueHolder sqrt(ValueHolder a) {
		// It will change the value in the holder (passed by-reference).
		squareRootHelper.sqrt(a);
		return a;
	}
}
