package example.tx.servlet;

import java.io.IOException;

import javax.ejb.EJB;
import javax.ejb.EJBException;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import example.tx.Messages;
import example.tx.Messages.ErrorType;
import example.tx.exceptions.MessageException;

@WebServlet("/stateless")
public class StatelessTest extends HttpServlet {
	private static final long serialVersionUID = 1L;

	@EJB(beanName = "StatelessMessagesBean")
	private Messages messages;

	@Override
	protected void doGet(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {
		// List previous messages
		request.setAttribute("messages", messages.getMessages());
		request.getRequestDispatcher("/WEB-INF/messages.jsp").forward(request,
				response);

		// Clear session
		request.getSession().removeAttribute("error");
	}

	@Override
	protected void doPost(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {
		String textParam = request.getParameter("text");
		String errorTypeParam = request.getParameter("errorType");
		HttpSession session = request.getSession();

		if (null != textParam && !"".equals(textParam)
				&& null != errorTypeParam) {
			ErrorType errorType;
			try {
				errorType = ErrorType.valueOf(errorTypeParam);
			} catch (Exception e) {
				errorType = ErrorType.None;
			}

			try {
				messages.add(textParam, errorType);
			} catch (MessageException e) {
				session.setAttribute("error", e.getMessage());
			} catch (EJBException e) {
				String error;
				if (null != e.getCause()) {
					// Runtime exceptions are automatically wrapped
					error = e.getCause().getMessage();
				} else {
					error = e.getMessage();
				}
				session.setAttribute("error", "EJBException: " + error);
			}
		}

		response.sendRedirect(request.getRequestURI());
	}
}
